#!/usr/local/bin/tclsh7.6
#this file is a quick hack to convert ns trace files into new nam tracefiles
#usage ns_to_nam <tracefile> <layoutfile>

set tracefilename [lindex $argv 0]
set layoutfilename [lindex $argv 1]
source $layoutfilename
set file [open $tracefilename "r"]

proc net {args} {
    switch [lindex $args 0] {
	"node" {
	    puts "n -t* -s[lindex $args 1] -SUP -v[lindex $args 2] -cblack"
	}
	"queue" {
	    puts "q -t* -s[lindex $args 1] -d[lindex $args 2] -a[lindex $args 3]"
	}
	"color" {
	    puts "c -t* -i[lindex $args 1] -n[lindex $args 2]"
	}
    }
}

proc mklink {dummy src dst speed delay dir} {
    global linktab
    set linktab($src,$dst,speed) [parse_speed $speed]
    set linktab($dst,$src,speed) [parse_speed $speed]
    set linktab($src,$dst,delay) [parse_delay $delay]
    set linktab($dst,$src,delay) [parse_delay $delay]
    puts "l -t* -s$src -d$dst -SUP -r$speed -D$delay -o$dir"
}

proc is_link {src dst} {
    global linktab
    return [info exists linktab($src,$dst,speed)]
}

proc parse_speed {speed} {
    set speed [string tolower $speed]
    set value $speed
    if {[set ix [string first mb $speed]]>=0} {
	set value [expr [string range $speed 0 [expr $ix-1]]*1000000]
    } elseif {[set ix [string first kb $speed]]>=0} {
	set value [expr [string range $speed 0 [expr $ix-1]]*1000]
    } elseif {[set ix [string first b $speed]]>=0} {
	set value [string range $speed 0 [expr $ix-1]]
    }
    return $value
}

proc parse_delay {delay} {
    set value $delay
    if {[set ix [string first us $delay]]>=0} {
	set value [expr [string range $delay 0 [expr $ix-1]]/1000000.0]
    } elseif {[set ix [string first ms $delay]]>=0} {
	set value [expr [string range $delay 0 [expr $ix-1]]/1000.0]
    } elseif {[set ix [string first s $delay]]>=0} {
	set value [string range $delay 0 [expr $ix-1]]
    }
#    puts $delay->$value
    return $value
}

set pid 0
proc packet_in_flight {time src dst type size id attr} {    
    global pid packets ptab linktab
    set atime [expr $time + $linktab($src,$dst,delay) + ($size*8.0)/$linktab($src,$dst,speed)]
    set ptab($pid) $atime
    set packets($pid,src) $src
    set packets($pid,dst) $dst
    set packets($pid,type) $type
    set packets($pid,size) $size
    set packets($pid,id) $id
#    puts "set packets($id,pid) $pid"
    set packets($id,pid) $pid
    set packets($pid,attr) $attr
    incr pid
}

proc check_flight {time} {
    global packets ptab
    set arrivals {}
    set times {}
#    puts "checking [array names ptab]"
    foreach p [array names ptab] {
#	puts "$p: $ptab($p) -s$packets($p,src) -d$packets($p,dst) -i$packets($p,id)"
	if {$time>=$ptab($p)} {
	    lappend arrivals $p
	    lappend times $ptab($p)
	}
    }
#    puts " -> $arrivals"
    lsort -real times
    foreach t $times {
	foreach p $arrivals {
	    if {[info exists ptab($p)]} {
		if {$ptab($p)==$t} {
		    puts "r -t$ptab($p) -s$packets($p,src) -d$packets($p,dst) -p$packets($p,type) -e$packets($p,size) -i$packets($p,id) -a$packets($p,attr)"
		    unset ptab($p)
		    unset packets($p,src)
		    unset packets($p,dst)
		    unset packets($p,type)
		    unset packets($p,size)
		    #XXX bug work around
		    catch "unset packets($packets($p,id),pid)"
		    unset packets($p,id)
		    unset packets($p,attr)
		}
	    }
	}
    }
}

proc drop_packet {id} {
    global packets ptab
    if {[info exists packets($id,pid)]} {
	set p $packets($id,pid)
	unset ptab($p)
	unset packets($p,src)
	unset packets($p,dst)
	unset packets($p,type)
	unset packets($p,size)
	unset packets($packets($p,id),pid)
	unset packets($p,id)
	unset packets($p,attr)
    }
}

nam_config net

while {[eof $file]==0} {
    set line [gets $file]
#    puts ">>$line"
    if {$line=={}} {break}
    set time [lindex $line 1]
    check_flight $time
    switch "X[lindex $line 0]" {
	"X+" {
	    set src [lindex $line 2]
	    set dst [lindex $line 3]
	    set type [lindex $line 4]
	    set size [lindex $line 5]
	    set flags [lindex $line 6]
	    set id [lindex $line 11]
	    set attr [lindex $line 7]
	    puts "+ -t$time -s$src -d$dst -p$type -e$size -i$id -a$attr"
	}
	"X-" {
	    set src [lindex $line 2]
	    set dst [lindex $line 3]
	    set type [lindex $line 4]
	    set size [lindex $line 5]
	    set flags [lindex $line 6]
	    set id [lindex $line 11]
	    set attr [lindex $line 7]
	    puts "- -t$time -s$src -d$dst -p$type -e$size -i$id -a$attr"
	    if {[is_link $src $dst]} {
		puts "h -t$time -s$src -d$dst -p$type -e$size -i$id -a$attr"
	    }
	    packet_in_flight $time $src $dst $type $size $id $attr
	}
	"Xd" {
	    set src [lindex $line 2]
	    set dst [lindex $line 3]
	    set type [lindex $line 4]
	    set size [lindex $line 5]
	    set flags [lindex $line 6]
	    set id [lindex $line 11]
	    set attr [lindex $line 7]
	    drop_packet $id
	    puts "d -t$time -s$src -d$dst -p$type -e$size -i$id -a$attr"

	}
	"Xh" {
	    set src [lindex $line 2]
	    set dst [lindex $line 3]
	    set type [lindex $line 4]
	    set size [lindex $line 5]
	    set flags [lindex $line 6]
	    set id [lindex $line 11]
	    set attr [lindex $line 7]
	    drop_packet $id
	    # puts "+ -t$time -s$src -d$dst -p$type -e$size -i$id -a$attr"
	    # puts "- -t$time -s$src -d$dst -p$type -e$size -i$id -a$attr"
	    puts "h -t$time -s$src -d$dst -p$type -e$size -i$id -a$attr"
	    packet_in_flight $time $src $dst $type $size $id $attr
	}
	"Xv" {
		puts $line
	}
    }
}